// DAYNBR.cpp : Defines the entry point for the console application.
//

#include "windows.h"

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>


int isleap( unsigned yr )
{
	return yr % 400 == 0 || (yr % 4 == 0 && yr % 100 != 0);
}

static unsigned months_to_days( unsigned month )
{
	return (month * 3057 - 3007) / 100;
}

static long years_to_days( unsigned yr )
{
	return yr * 365L + yr / 4 - yr / 100 + yr / 400;
}

long ymd_to_scalar( unsigned yr, unsigned mo, unsigned day )
{
	long scalar;
	scalar = day + months_to_days( mo );
	if( mo > 2 )                         /* adjust if past February */
		scalar -= isleap(yr) ? 1 : 2;
	yr--;
	scalar += years_to_days(yr);
	return scalar;
}

/*
**  Return the day of the year (1 - 365/6)
*/
int daynum( int year, int month, int day )
{
	long jan1date;
	jan1date = ymd_to_scalar(year, 1, 1);
	return (int)(ymd_to_scalar(year, month, day) - jan1date + 1L);
}


int main( int argc, char* argv[] )
{
	int i, nd_ref, dw, d, n;
	time_t tnow;
	struct tm *tmnow;
	char cmd_line[512];
	char par[256], *p, *pd;
	char sDayNum[16];
	
	// Calculate reference = number of days since 1 Jan of the current year
	time( &tnow );
	tmnow = localtime( &tnow );
	nd_ref = ymd_to_scalar( tmnow->tm_year + 1900, tmnow->tm_mon + 1, tmnow->tm_mday ) - ymd_to_scalar( tmnow->tm_year + 1900, 1, 1 ) + 1;
	dw = ymd_to_scalar( tmnow->tm_year + 1900, tmnow->tm_mon + 1, tmnow->tm_mday ) % 7;
#ifdef _DEBUG
	printf( "REF Day number = %d, day of week = %d\n", nd_ref, dw );
#endif

	if( argc <= 1 ) {
		printf( "DAYNBR Version 1 written by Bob - http://www.grsoftware.net\n" );
		printf( "USAGE:\n" );
		printf( "DAYNBR [/D<n> | /+<n> | /-<n>] dos_command\n" );
		printf( "   return the day of the week number\n" );
		printf( "   or the dos_command return value\n" );
		return dw;
	}

	for( argv++,argc-- ; argc > 0 && argv[0][0] == '/'; argc--,argv++ ) {
		switch( argv[0][1] ) {
		case 'D':
			i = atoi( &argv[0][2] ) % 7;
			d = ymd_to_scalar( tmnow->tm_year + 1900, tmnow->tm_mon + 1, tmnow->tm_mday );
			// Go backward until the same day of the week number is found
			while( d % 7 != i ) {
				d--;
				nd_ref--;
			}
			dw = d % 7;
			break;
		case '+':
			i = atoi( &argv[0][2] );
			nd_ref += i;
			break;
			
		case '-':
			i = atoi( &argv[0][2] );
			nd_ref -= i;
			break;

		default:
			break;
		}
	}

	*cmd_line = '\0';
	while( argc > 0 ) {
		*par = '\0';
		sprintf( sDayNum, "%3.3d", nd_ref );
		i = 0;
		
		p = argv[0];
		pd = par;
		while( *p ) {
			// Replace @@ with a single @
			if( *p == '@' ) {
				if( p[1] == '@' )
					*pd++ = '@';
				else {
					// This is the start of a triplet
					p++;	// skip the @
					n = 0;
					while( n < 3 ) {
						if( *p == '#' )
							*pd++ = sDayNum[n];
						else
							*pd++ = *p;

						p++;
						n++;
					}
				}
 			} else
				*pd++ = *p;

			p++;
		}
		*pd = '\0';
		
		// Add the parameter to the command line
		if( *cmd_line != '\0' )
			strcat( cmd_line, " " );
		strcat( cmd_line, par );
		argc--;
		argv++;
	}

#ifdef _DEBUG
	printf( "Updated Day number = %d, day of week = %d\n", nd_ref, dw );
#endif

	if( *cmd_line ) {
#ifdef _DEBUG		
		printf( "Execute %s\n", cmd_line );
#endif
		return system( cmd_line );
	}
	
	return dw;
}
